# Features

- [Features](#features)
  * [Resolve image on the fly](#resolve-image-on-the-fly)
    + [Definition](#definition)
    + [Usage & Result](#usage---result)

## Resolve image on the fly

### Definition

This plugin put in cache and resolve media choose by the browser through `<picture>`, `<source>` and `<img>` tags.

Documentation about `<source>` element is [here](https://developer.mozilla.org/fr/docs/Web/HTML/Element/Source)

### Usage & Result

We have a twig function :

`dedi_sylius_image_optimizer_for_image(imageName[, alternativeText, runtimeConfigurationImage, yourOwnTemplate])` : 

1. this function inject `<source>` elements for the 3 mosts `Viewport` must known sizes (`webp` compatiblity is handled); 
2. browser chooses the more optimized format after the page has been loading and do a request to the server:
3  server resize image with option that the browser send to us (we don't upscale them)
4. caching media is handle ny server
5. image is display;
6. CSS could work with the format appropriate for the size screen.

```twig
{{ dedi_sylius_image_optimizer_for_image(imageName) }}
```

#### With runtimeConfigurationImage

By default, we use this template : [src/Resources/views/Shop/Media/picture_optimizer.html.twig](src/Resources/views/Shop/Media/picture_optimizer.html.twig)

If `runtimeConfigurationImage` is define, we use [src/Resources/views/Shop/Media/picture_optimizer_custom.html.twig](src/Resources/views/Shop/Media/picture_optimizer_custom.html.twig)

For exemple:
- 1 media queries and 1 format for small viewport device (768px);
- 1 media queries and 1 format for middle viewport device (1200px);
- 1 media queries and 1 format for big viewport device (1920px).

**IMPORTANT: the order is important because of the media queries like in CSS**

Example:

```twig
    {%  set runtimeConfigurationImage = [
        {
            'media': '(max-width: 767px)',
            'sizes' : '(max-width: 1534px) 100vw, 768px',
            'srcset':
            [
                {'width':'768', 'height': '534'},
            ]
        },
        {
            'media': '(min-width: 768px) and (max-width: 1199px)',
            'sizes' : '(max-width: 2400px) 100vw, 1200px',
            'srcset':
            [
                {'width':'1200', 'height': '798'},
            ]
        },
        {
            'sizes' : '(max-width: 3488px) 100vw, 1920px',
            'srcset':
            [
                {'width':'1920', 'height': '1276'},
            ]
        },
    ]
    %}

   {{ dedi_sylius_image_optimizer_for_image(imageName, alternativeText|default(''), runtimeConfigurationImage) }}
```

The result should be like this:

```html
    <picture class="media-object">
        <source media="(max-width: 767px)" sizes="(max-width: 1534px) 100vw, 768px" srcset="/static/app_webp_generic/robesplies_detourees1copie2-5ee0d76fd051c.png?width=768&amp;height=534 768w" type="image/webp">
        <source media="(min-width: 768px) and (max-width: 1199px)" sizes="(max-width: 2400px) 100vw, 1200px" srcset="/static/app_webp_generic/robesplies_detourees1copie2-5ee0d76fd051c.png?width=1200&amp;height=798 1200w" type="image/webp">
        <source sizes="(max-width: 3488px) 100vw, 1920px" srcset="/static/app_webp_generic/robesplies_detourees1copie2-5ee0d76fd051c.png?width=1920&amp;height=1276 1920w" type="image/webp">
        <source media="(max-width: 767px)" sizes="(max-width: 1534px) 100vw, 768px" srcset="/static/app_generic/robesplies_detourees1copie2-5ee0d76fd051c.png?width=768&amp;height=534 768w">
        <source media="(min-width: 768px) and (max-width: 1199px)" sizes="(max-width: 2400px) 100vw, 1200px" srcset="/static/app_generic/robesplies_detourees1copie2-5ee0d76fd051c.png?width=1200&amp;height=798 1200w">
        <img sizes="(max-width: 3488px) 100vw, 1920px" srcset="/static/app_generic/robesplies_detourees1copie2-5ee0d76fd051c.png?width=1920&amp;height=1276 1920w" src="/static/app_generic/robesplies_detourees1copie2-5ee0d76fd051c.png?width=1920&amp;height=1276" alt="La Douceur de Laine">
    </picture>
```

#### Use your own template

The last parameter allows you to defined your own template like this:

```twig
   {{ dedi_sylius_image_optimizer_for_image(imageName, alternativeText|default(''), null, '@App/Path/To/custumer_template.html.twig') }}
```
